@@ -1,26 +1,33 @@
 import 'flexlayout-react/style/light.css';
-import Toolbar from '@mui/material/Toolbar';
 import * as FlexLayout from 'flexlayout-react';
 import React, { Component } from 'react';
 
-import { JUPYTER_BUTTON_CLASS, IDict } from './utils';
+import {
+  JUPYTER_BUTTON_CLASS,
+  IDict,
+  MESSAGE_ACTION,
+  downloadString,
+} from './utils';
 import { WidgetMenu } from './menuWidget';
 import { WidgetWrapper } from './widgetWrapper';
+import Header from './headerComponent';
 import {
   defaultModelFactoty,
   ILayoutConfig,
   updateModelEditable,
 } from './defaultModelFactory';
-import dialogBody from './dialogWidget';
+import { dialogBody } from './dialogWidget';
 import { showDialog } from '@jupyterlab/apputils';
 import { ContextMenu } from '@lumino/widgets';
 import { CommandRegistry } from '@lumino/commands';
+import { uuid } from '@jupyter-widgets/base';
 
 interface IProps {
   send_msg: ({ action: string, payload: any }) => void;
   model: any;
   style: IDict;
   editable: boolean;
+  header: boolean | IDict;
 }
 
 interface IState {
@@ -28,13 +35,16 @@ interface IState {
   defaultOuterModel: IDict;
   defaultModel: IDict;
   widgetList: Array<string>;
+  placeholderList: Array<string>;
+  factoryDict: IDict<IDict>;
   editable: boolean;
+  header: boolean | IDict;
 }
 
 export class FlexWidget extends Component<IProps, IState> {
   constructor(props: IProps) {
     super(props);
-    props.model.listenTo(props.model, 'msg:custom', this.on_msg);
+
     this.innerlayoutRef = {};
     this.layoutConfig = props.model.get('layout_config') as ILayoutConfig;
 
@@ -61,42 +71,114 @@ export class FlexWidget extends Component<IProps, IState> {
       );
       flexModel = FlexLayout.Model.fromJson(defaultOuterModel as any);
     }
-
     this.state = {
       model: flexModel,
       defaultOuterModel,
       defaultModel,
       widgetList: Object.keys(this.props.model.get('children')),
+      placeholderList: [],
+      factoryDict: this.props.model.get('widget_factories'),
       editable: props.editable,
+      header: props.header,
     };
     this.model = props.model;
-    this.contextMenuCache = new Map<string, ContextMenu>();
+    this.innerContextMenuCache = new Map<string, ContextMenu>();
   }
 
+  componentDidMount(): void {
+    this.props.model.listenTo(this.props.model, 'msg:custom', this.on_msg);
+    this.props.model.listenTo(
+      this.props.model,
+      'change:placeholder_widget',
+      this.on_placeholder_change
+    );
+    this.uuid = uuid();
+    this.props.send_msg({
+      action: MESSAGE_ACTION.REGISTER_FRONTEND,
+      payload: { uuid: this.uuid },
+    });
+  }
+
+  on_placeholder_change = (
+    model: any,
+    newValue: Array<string>,
+    change: IDict
+  ): void => {
+    this.setState((old) => ({
+      ...old,
+      placeholderList: newValue,
+    }));
+  };
+
   on_msg = (data: { action: string; payload: any }, buffer: any[]): void => {
     const { action, payload } = data;
     switch (action) {
-      case 'update_children':
-        {
-          const wName: string = payload.name;
+      case MESSAGE_ACTION.UPDATE_CHILDREN: {
+        const wName: string = payload.name;
+        if (
+          !this.state.widgetList.includes(wName) &&
+          !this.state.placeholderList.includes(wName)
+        ) {
           this.setState((old) => ({
             ...old,
             widgetList: [...old.widgetList, wName],
           }));
         }
+        break;
+      }
 
-        return null;
+      case MESSAGE_ACTION.SAVE_TEMPLATE_FROM_PYTHON: {
+        const fileName: string = payload.name;
+        const uuid: string = payload.uuid;
+        if (uuid === this.uuid && fileName) {
+          if (!fileName) {
+            alert('Invalid file name!');
+            return;
+          }
+          this.props.send_msg({
+            action: MESSAGE_ACTION.SAVE_TEMPLATE,
+            payload: {
+              file_name: fileName,
+              json_data: this.state.model.toJson(),
+            },
+          });
+        }
+        break;
+      }
+
+      case MESSAGE_ACTION.LOAD_TEMPLATE_FROM_PYTHON: {
+        const template = payload.template;
+        this.setState((old) => ({
+          ...old,
+          model: FlexLayout.Model.fromJson(template),
+        }));
+        break;
+      }
     }
   };
   factory = (node: FlexLayout.TabNode): JSX.Element => {
     const component = node.getComponent() as 'Widget' | 'sub';
-    // const config = node.getConfig();
+    const config = node.getConfig();
     const nodeId = node.getId();
     const name = node.getName();
-
     switch (component) {
       case 'Widget': {
-        return <WidgetWrapper model={this.model} widgetName={name} />;
+        const selector = '.ipyflex-widget-box';
+        const contextMenu = this.individualContextMenuFactory(
+          node.getParent().getId(),
+          node.getParent().getModel(),
+          selector
+        );
+        return (
+          <WidgetWrapper
+            model={this.model}
+            widgetName={name}
+            factoryDict={this.state.factoryDict}
+            send_msg={this.props.send_msg}
+            extraData={config.extraData}
+            contextMenu={contextMenu}
+          />
+        );
       }
       case 'sub': {
         return this.generateSection(node, nodeId);
@@ -160,6 +242,21 @@ export class FlexWidget extends Component<IProps, IState> {
         onAction={(action: FlexLayout.Action) =>
           this.innerOnAction(nodeId, action)
         }
+        onContextMenu={(
+          node:
+            | FlexLayout.TabNode
+            | FlexLayout.TabSetNode
+            | FlexLayout.BorderNode,
+          event: React.MouseEvent<HTMLElement, MouseEvent>
+        ) => {
+          if (event.shiftKey) {
+            return;
+          }
+          event.preventDefault();
+          event.stopPropagation();
+          const contextMenu = this.innerContextMenuCache.get(node.getId());
+          contextMenu.open(event.nativeEvent);
+        }}
       />
     );
   };
@@ -184,10 +281,17 @@ export class FlexWidget extends Component<IProps, IState> {
     outerNodeID: string,
     action: FlexLayout.Action
   ): FlexLayout.Action => {
+    if (action.type === 'FlexLayout_DeleteTabset') {
+      const id: string = action.data.node;
+      if (this.innerContextMenuCache.has(id)) {
+        this.innerContextMenuCache.delete(id);
+      }
+    }
     if (
       action.type === 'FlexLayout_MoveNode' ||
       action.type === 'FlexLayout_AdjustSplit' ||
       action.type === 'FlexLayout_DeleteTab' ||
+      action.type === 'FlexLayout_DeleteTabset' ||
       action.type === 'FlexLayout_MaximizeToggle' ||
       action.type === 'FlexLayout_SelectTab'
     ) {
@@ -206,21 +310,34 @@ export class FlexWidget extends Component<IProps, IState> {
     },
     nodeId: string
   ): void => {
+    const tabsetId = tabSetNode.getId();
+    const selector = '.flexlayout__tabset_tabbar_inner_top';
+    if (!this.innerContextMenuCache.has(tabsetId)) {
+      const contextMenu = this.individualContextMenuFactory(
+        tabsetId,
+        tabSetNode.getModel(),
+        selector
+      );
+
+      this.innerContextMenuCache.set(tabsetId, contextMenu);
+    }
     if (this.state.editable) {
-      const tabsetId = tabSetNode.getId();
       renderValues.buttons.push(
         <WidgetMenu
           widgetList={this.state.widgetList}
+          placeholderList={this.state.placeholderList}
+          factoryDict={this.state.factoryDict}
           nodeId={nodeId}
           tabsetId={tabsetId}
-          addTabToTabset={(name: string) => {
+          addTabToTabset={(name: string, extraData?: IDict) => {
             this.innerlayoutRef[nodeId].current.addTabToTabSet(tabsetId, {
               component: 'Widget',
               name: name,
-              config: { layoutID: nodeId },
+              config: { layoutID: nodeId, extraData: extraData },
             });
           }}
           model={this.props.model}
+          send_msg={this.props.send_msg}
         />
       );
     }
@@ -271,7 +388,7 @@ export class FlexWidget extends Component<IProps, IState> {
       const fileName = result.value;
       if (fileName) {
         this.props.send_msg({
-          action: 'save_template',
+          action: MESSAGE_ACTION.SAVE_TEMPLATE,
           payload: {
             file_name: result.value,
             json_data: this.state.model.toJson(),
@@ -283,19 +400,45 @@ export class FlexWidget extends Component<IProps, IState> {
     }
   };
 
+  exportTemplate = async (): Promise<void> => {
+    const result = await showDialog<string>(
+      dialogBody('Export template', null, 'Export')
+    );
+    if (result.button.label === 'Export') {
+      let fileName = result.value;
+      if (fileName) {
+        fileName = fileName.replace('.json', '') + '.json';
+        const jsonString = this.state.model.toString();
+        downloadString(jsonString, 'application/json', fileName);
+      } else {
+        alert('Invalid file name!');
+      }
+    }
+  };
+
+  importTemplate = (content: string): void => {
+    const model = JSON.parse(content);
+    this.setState((old) => ({
+      ...old,
+      model: FlexLayout.Model.fromJson(model),
+    }));
+  };
+
   toggleLock = (): void => {
     this.setState((old) => ({ ...old, editable: !old.editable }));
   };
 
-  contextMenuFactory = (node: FlexLayout.Node): ContextMenu => {
+  individualContextMenuFactory = (
+    tabsetId: string,
+    model: FlexLayout.Model,
+    selector: string
+  ): ContextMenu => {
     const commands = new CommandRegistry();
-    const nodeId = node.getId();
     commands.addCommand('hide-tab-bar', {
       execute: () => {
-        const subLayout = this.innerlayoutRef[nodeId].current;
-        subLayout.props.model.doAction(
-          FlexLayout.Actions.updateModelAttributes({
-            tabSetEnableTabStrip: false,
+        model.doAction(
+          FlexLayout.Actions.updateNodeAttributes(tabsetId, {
+            enableTabStrip: false,
           })
         );
       },
@@ -304,10 +447,9 @@ export class FlexWidget extends Component<IProps, IState> {
     });
     commands.addCommand('show-tab-bar', {
       execute: () => {
-        const subLayout = this.innerlayoutRef[nodeId].current;
-        subLayout.props.model.doAction(
-          FlexLayout.Actions.updateModelAttributes({
-            tabSetEnableTabStrip: true,
+        model.doAction(
+          FlexLayout.Actions.updateNodeAttributes(tabsetId, {
+            enableTabStrip: true,
           })
         );
       },
@@ -317,24 +459,55 @@ export class FlexWidget extends Component<IProps, IState> {
     const contextMenu = new ContextMenu({ commands });
     contextMenu.addItem({
       command: 'show-tab-bar',
-      selector: '.flexlayout__tab_button_bottom',
+      selector: this.props.editable ? selector : 'null',
       rank: 0,
     });
     contextMenu.addItem({
       command: 'hide-tab-bar',
-      selector: '.flexlayout__tab_button_bottom',
+      selector: this.props.editable ? selector : 'null',
       rank: 1,
     });
     return contextMenu;
   };
 
   render(): JSX.Element {
+    let headerHeight = 0;
+    if (this.state.header) {
+      headerHeight = 35;
+    }
+    const mainHeight = `calc(100% - 5px - ${headerHeight}px)`;
+    let titleProps = {};
+    if (this.state.header && !(typeof this.state.header === 'boolean')) {
+      titleProps = { ...this.state.header };
+      if (!titleProps['buttons']) {
+        titleProps['buttons'] = [];
+      }
+      if (titleProps['buttons'] === true) {
+        titleProps['buttons'] = ['save', 'export', 'import'];
+      }
+      if (!this.props.editable) {
+        titleProps['buttons'] = (titleProps['buttons'] as Array<string>).filter(
+          (btn) => btn !== 'save'
+        );
+      }
+    }
     return (
       <div style={{ height: '510px', ...this.props.style }}>
+        {this.state.header ? (
+          <Header
+            {...titleProps}
+            saveTemplate={this.saveTemplate.bind(this)}
+            exportTemplate={this.exportTemplate.bind(this)}
+            importTemplate={this.importTemplate.bind(this)}
+          />
+        ) : (
+          <div />
+        )}
         <div
           style={{
             width: '100%',
-            height: this.state.editable ? 'calc(100% - 31px)' : '100%',
+            height: mainHeight,
+            position: 'relative',
           }}
         >
           <FlexLayout.Layout
@@ -363,45 +536,8 @@ export class FlexWidget extends Component<IProps, IState> {
             ) => {
               this.onRenderOuterTabSet(tabSetNode, renderValues);
             }}
-            onRenderTab={(node, _) => {
-              const nodeId = node.getId();
-              if (!this.contextMenuCache.has(nodeId)) {
-                const contextMenu = this.contextMenuFactory(node);
-                this.contextMenuCache.set(nodeId, contextMenu);
-              }
-            }}
-            onContextMenu={(
-              node:
-                | FlexLayout.TabNode
-                | FlexLayout.TabSetNode
-                | FlexLayout.BorderNode,
-              event: React.MouseEvent<HTMLElement, MouseEvent>
-            ) => {
-              event.preventDefault();
-              event.stopPropagation();
-              const contextMenu = this.contextMenuCache.get(node.getId());
-              contextMenu.open(event.nativeEvent);
-            }}
           />
         </div>
-        {this.state.editable ? (
-          <Toolbar
-            variant="dense"
-            style={{
-              height: '30px',
-              minHeight: '30px',
-            }}
-          >
-            <button
-              className={JUPYTER_BUTTON_CLASS}
-              onClick={this.saveTemplate}
-            >
-              Save template
-            </button>
-          </Toolbar>
-        ) : (
-          <div></div>
-        )}
       </div>
     );
   }
@@ -410,8 +546,8 @@ export class FlexWidget extends Component<IProps, IState> {
   private innerlayoutRef: { [key: string]: React.RefObject<FlexLayout.Layout> };
   private model: any;
   private layoutConfig: ILayoutConfig;
-  // private contextMenu: ContextMenu;
-  private contextMenuCache: Map<string, ContextMenu>;
+  private innerContextMenuCache: Map<string, ContextMenu>;
+  private uuid: string;
 }
 
 export default FlexWidget;
